@@ -263,33 +263,15 @@
 #include "config.h"
 #include "gtkactivatable.h"
 #include "gtkactiongroup.h"
-#include "gtktypeutils.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
 
-static void gtk_activatable_class_init (gpointer g_iface);
-
-GType
-gtk_activatable_get_type (void)
-{
-  static GType activatable_type = 0;
-
-  if (!activatable_type) {
-    activatable_type =
-      g_type_register_static_simple (G_TYPE_INTERFACE, I_("GtkActivatable"),
-				     sizeof (GtkActivatableIface),
-				     (GClassInitFunc) gtk_activatable_class_init,
-				     0, NULL, 0);
-
-    g_type_interface_add_prerequisite (activatable_type, G_TYPE_OBJECT);
-  }
-
-  return activatable_type;
-}
+typedef GtkActivatableIface GtkActivatableInterface;
+G_DEFINE_INTERFACE (GtkActivatable, gtk_activatable, G_TYPE_OBJECT)
 
 static void
-gtk_activatable_class_init (gpointer g_iface)
+gtk_activatable_default_init (GtkActivatableInterface *iface)
 {
   /**
    * GtkActivatable:related-action:
@@ -302,7 +284,7 @@ gtk_activatable_class_init (gpointer g_iface)
    *
    * Since: 2.16
    */
-  g_object_interface_install_property (g_iface,
+  g_object_interface_install_property (iface,
 				       g_param_spec_object ("related-action",
 							    P_("Related Action"),
 							    P_("The action this activatable will activate and receive updates from"),
@@ -325,7 +307,7 @@ gtk_activatable_class_init (gpointer g_iface)
    *
    * Since: 2.16
    */
-  g_object_interface_install_property (g_iface,
+  g_object_interface_install_property (iface,
 				       g_param_spec_boolean ("use-action-appearance",
 							     P_("Use Action Appearance"),
 							     P_("Whether to use the related actions appearance properties"),
@@ -495,7 +477,7 @@ gtk_activatable_do_set_related_action (GtkActivatable *activatable,
  *
  * Gets the related #GtkAction for @activatable.
  *
- * Returns: the related #GtkAction if one is set.
+ * Returns: (transfer none): the related #GtkAction if one is set.
  *
  * Since: 2.16
  **/
